! $Id: allblackbodies.prg,v 1.2 1999/08/13 09:33:34 cguirao Exp $
! $Name: FIASCO-1_3 $
!
! AUTHOR: C. Guirao
!
! NAME: allblackbodies.prg
!   Procedure that generates a plot with the blackbodies curves at
!   different temperatures between 400nm and 700nm
!
! INPUT:
!
! OUTPUT:
!   A postscript file allblackbodies.ps
!
! CONVENTION:
!   Physical Constants (c.g.s units)
!   c -> Speed of light
!   h -> Planck's constant
!   k -> Bolzmann's constant
!
 DEFINE/LOCA c/d/1/1 3.0e10
 DEFINE/LOCA h/d/1/1 6.63e-27
 DEFINE/LOCA k/d/1/1 1.38e-16
 DEFINE/LOCA reply/c/1/1  y
 DEFINE/LOCA printer/c/1/40      "postscript"
!
 DEFINE/LOCA t1/d/1/1 3.0e3
 DEFINE/LOCA t2/d/1/1 4.0e3
 DEFINE/LOCA t3/d/1/1 4.5e3
 DEFINE/LOCA t4/d/1/1 5.0e3
 DEFINE/LOCA t5/d/1/1 5.5e3
 DEFINE/LOCA t6/d/1/1 6.0e3
 DEFINE/LOCA t7/d/1/1 7.0e3
 DEFINE/LOCA t8/d/1/1 8.0e3
 DEFINE/LOCA t9/d/1/1 10.0e3
 DEFINE/LOCA t10/d/1/1 20.0e3
 DEFINE/LOCA t11/d/1/1 40.0e3
!
 CREATE/TABL &t 30 30
 CREATE/COLU &t :Wavelength R*8 E12.6 "nm"
 CREATE/COLU &t :Flux R*8 F12.6 "Bv(T)"
 COMPUT/TABL &t :W = (seq+40)*1e-6
 COMPUT/TABL &t :Wavelength = (seq+40)*10
 COMPUT/TABL &t :F = (2*{h}*{c}**2)/(:W**5)
 COMPUT/TABL &t :F1 = :F/(EXP(({h}*{c})/(:W*{k}*{t1})) - 1)*1e-14
 COMPUT/TABL &t :F2 = :F/(EXP(({h}*{c})/(:W*{k}*{t2})) - 1)*1e-14
 COMPUT/TABL &t :F3 = :F/(EXP(({h}*{c})/(:W*{k}*{t3})) - 1)*1e-14
 COMPUT/TABL &t :F4 = :F/(EXP(({h}*{c})/(:W*{k}*{t4})) - 1)*1e-14
 COMPUT/TABL &t :F5 = :F/(EXP(({h}*{c})/(:W*{k}*{t5})) - 1)*1e-14
 COMPUT/TABL &t :F6 = :F/(EXP(({h}*{c})/(:W*{k}*{t6})) - 1)*1e-14
 COMPUT/TABL &t :F7 = :F/(EXP(({h}*{c})/(:W*{k}*{t7})) - 1)*1e-14
 COMPUT/TABL &t :F8 = :F/(EXP(({h}*{c})/(:W*{k}*{t8})) - 1)*1e-14
 COMPUT/TABL &t :F9 = :F/(EXP(({h}*{c})/(:W*{k}*{t9})) - 1)*1e-14
 COMPUT/TABL &t :F10 = :F/(EXP(({h}*{c})/(:W*{k}*{t10})) - 1)*1e-14
 COMPUT/TABL &t :F11 = :F/(EXP(({h}*{c})/(:W*{k}*{t11})) - 1)*1e-14
!
!
 SET/GRAPH pm=0 font=1 yoffset=22 lwidth=2 tsize=1.5 xa=auto ya=auto
 SET/GRAP stype=0 ltype=1
 CREATE/GRAP
!
 SET/GRAP pm=0
 PLOT/AXES 390,720 0.001,0.079
 OVERPLOT/TABL &t :Wavelength :F1
!
 OVERPLOT/AXES 390,720 0.15,0.43
 OVERPLOT/TABL &t :Wavelength :F2
!
 OVERPLOT/AXES 390,720 0.4,0.78
 OVERPLOT/TABL &t :Wavelength :F3
!
 OVERPLOT/AXES 390,720 0.89,1.29
 OVERPLOT/TABL &t :Wavelength :F4
!
 OVERPLOT/AXES 390,720 1.71,2.078
 OVERPLOT/TABL &t :Wavelength :F5
!
 OVERPLOT/AXES 390,720 2.32,3.22
 OVERPLOT/TABL &t :Wavelength :F6
!
 OVERPLOT/AXES 390,720 3.72,7.04
 OVERPLOT/TABL &t :Wavelength :F7
!
 OVERPLOT/AXES 390,720 5.3,13.5
 OVERPLOT/TABL &t :Wavelength :F8
!
 OVERPLOT/AXES 390,720 9,32.5
 OVERPLOT/TABL &t :Wavelength :F9
!
 OVERPLOT/AXES 390,720 29,227
 OVERPLOT/TABL &t :Wavelength :F10
!
 OVERPLOT/AXES 390,720 60,780
 OVERPLOT/TABL &t :Wavelength :F11
!
 SET/GRAP pm=1
 OVERPL/AXES 390,720 0,10
!
 SET/GRAPH font=1 tsize=1 
 LABEL/GRAPH "3000K" ? 32
 LABEL/GRAPH "4000K" ? 48
 LABEL/GRAPH "4500K" ? 50
 LABEL/GRAPH "5000K" ? 60 
 LABEL/GRAPH "5500K" ? 66
 LABEL/GRAPH "6000K" ?
 LABEL/GRAPH "7000K" ? -44
 LABEL/GRAPH "8000K" ? -40
 LABEL/GRAPH "10000K" ? -30
 LABEL/GRAPH "20000K" ? -25
 LABEL/GRAPH "40000K" ? -22
!
 SET/GRAPH font=1 tsize=1.5 
 LABEL/GRAPH "Wavelength (nm)"
 LABEL/GRAPH "Relative Flux" ? 90
!
 askToContinue:
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "Creating a postscript file [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   COPY/GRAPH {printer}
   WRITE/OUT "Converting plot to postscript file allblackbodies.ps"
   $mv {printer}.ps allblackbodies.ps
   GOTO continue
 ELSEIF reply(:1) .eq. "n" THEN
   GOTO continue
 ELSE
   GOTO askToContinue
 ENDIF
 continue:
!
 askRemove:
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "Remove all temporary files [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO remove
 ELSEIF reply(:1) .eq. "n" THEN
   ECHO/OFF
   RETURN/EXIT
 ELSE
   GOTO askRemove
 ENDIF
 remove:
!
! Clean all temporary files
!
 WRITE/OUT "Cleaning temporary files and finishing"
 $rm -f graph_wn*.plt
 $rm -f middummt.tbl
 ECHO/OFF
