! $Id: averaging.prg,v 1.3 1999/08/14 09:16:43 cguirao Exp $
! $Name: FIASCO-1_3 $
!
! AUTHOR: C.Guirao
!
! NAME: averaging.prg
!   Creates a master frame by averaging a collection of FITS/BDF images
!   Used to create master bias, master dark current and master flat fields
!
! INPUT: 
!   - P1: Set of FITS/BDF frames to be averaged (wild chars can be used)
!
! OUTPUT: 
!   - P2: The averaged image in BDF format
!
!**********************************************************************       
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS/BDF frames to be averaged.
! p2 REQUIRED: name of BDF averaged frame to be created.
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ?         IMA "Enter FITS/BDF frames to be averaged: "
 DEFINE/PARA p2 ?         IMA "Enter name of BDF output: "
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20  	"averaging"
 DEFINE/LOCA output/c/1/40   	{p2}
 DEFINE/LOCA fitsframe/c/1/40
 DEFINE/LOCA frame/c/1/40
 DEFINE/LOCA bias/c/1/40        "master_bias.bdf"
 DEFINE/LOCA catal/i/1/1 	0
 DEFINE/LOCA reply/c/1/1        y
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10         "" ? +lower_levels
!
! Create an empty catalog where to save BDF frames after bias subtraction
!
 CREATE/ICAT master_b.cat null
!
! Create a catalog with all input FITS files
!
 CREATE/FCAT master.cat {p1} 
!
! Initiate contex LONG required for COMBINE/LONG
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT long >NULL
 ENDIF
!
! For each input FITS file
!
 LOOP:
!
   STORE/FRAM in_b master.cat 1 FINITO
   !
   WRITE/KEY fitsframe {in_b} 
   !
   @@ file_ext {fitsframe}
   frame = basename
   !
   ! Convert FITS format into MIDAS BDF format
   !
    IF ext .ne. ".bdf" THEN
      @@ compute_fits {fitsframe} {frame}
    ENDIF
   !
   ! Bias subtraction
   !
   IF m$exist(bias) .eq. 1 THEN
     COMPUTE/IMA {frame}b = {frame} - {bias}
     DELETE/IMA {frame} NO
     frame = frame + "b"
     WRITE/OUT "{cmd}: creating {frame} with {bias} subtracted"
   ELSE
     WRITE/OUT "{cmd}: {bias} not found. No bias substructed"
   ENDIF
   !
   ! Add the 1-dim frame to the catalog
   !
   ADD/ICAT master_b.cat {frame} >NULL
   !
   GOTO LOOP
 FINITO:
!
! Combine all bias subtracted images in the master_b.cat catalog and 
! generate a median average
!
 COMBINE/LONG master_b.cat {output} >NULL
 WRITE/OUT "{cmd}: {output} created as the median average"
!
! Clean all temporary files
!
 WRITE/OUT "{cmd}: cleaning temporary files"
 IF ext .ne. ".bdf" THEN
   DELETE/ICAT master_b.cat Noconf >NULL
 ENDIF
 $rm -f master_b.cat
 $rm -f master.cat
 $rm -f lntab.tbl
 $rm -f dirfile.ascii
 $rm -f NULL
!
ECHO/OFF
