! $Id: compute_flux.prg,v 1.1 2002/01/30 09:12:32 cguirao Exp $
! $Name: FIASCO-1_3 $ 
!
! AUTHOR: C.Guirao
!
! NAME: computer_flux.prg
!   Procedure to calculate the total intensity (flux integration) 
!   of emission lines
!
!   If the input file still includes bias & dark current, it can be 
!   calculated in the input file as a constant. E.g.: 
!   In order to remove bias & dark-current, lets calculate
!   the median value IMAGE in an area of the CCD not used by the spectrum
!
!               ##########################
!               # ---------------------- #
!               # |                    | #
!               # |                    | #  <--- dark_area(1) x dark_area(2) 
!               # ---------------------- #
!               #========================#
!               #                        #
!               #                        #
!               #                        #
!               ##########################
!
! INPUT: 
!   - FITS/BDF image with emission lines 
!   - OPTIONAL: Size of the integration area (default: 15,15)
!
! OUTPUT: 
!
!**********************************************************************       
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS/BDF image
! p2 OPTIONAL: size of the integration area to compute flux (total intensity)
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ?         IMA "Enter FITS/BDF image:"
 DEFINE/PARA p2 0 ?       NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20		"INFO"
 DEFINE/LOCA frame/c/1/40	{p1}
 DEFINE/LOCA name/c/1/40
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA original/c/1/40
 DEFINE/LOCA size_area/i/1/2	{p2}   
 DEFINE/LOCA coord/i/1/2
 DEFINE/LOCA dark_area/i/1/2
 DEFINE/LOCA size_ccd/r/1/2
 DEFINE/LOCA stat_dark/r/1/1 	0.
 DEFINE/LOCA reply/c/1/1        y
!
 DEFINE/LOCA ext/c/1/10 
 DEFINE/LOCA indx/i/1/1         0
 DEFINE/LOCA name_bdf/c/1/40
 DEFINE/LOCA name_fit/c/1/40
!
 name = frame
!
! From file_ext.prg
!
! Gets basename and extension of input file
!
 indx = m$index(name,".")
 IF indx .ne. 0 THEN
   IF m$exist(name) .eq. 0 THEN
     WRITE/OUT "ERROR: File {name} NOT FOUND"
     RETURN/EXIT
   ENDIF
   ext = name({indx}:)
   indx = indx-1
   basename = name(1:{indx})
 ELSE
   basename = name
   ext = ".bdf"
   name_bdf = basename + ".bdf"
   IF m$exist(name_bdf) .eq. 0 THEN
     ext = ".fit"
     name_fit = basename + ".fit"
     IF m$exist(name_fit) .eq. 0 THEN
       WRITE/OUT "ERROR: Neither file {name}.fit nor {name}.bdf FOUND"
       RETURN/EXIT
     ENDIF
   ENDIF
 ENDIF
!
! WRITE/OUT "basename= {basename}"
! WRITE/OUT "ext= {ext}"
 original =  basename
 frame = original + "_"
!
!
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
  WRITE/OUT "{cmd}: Converting {original}{ext} into {frame}.bdf"
  INDISK/FITS {original}{ext} {frame} >NULL
 ELSE
  WRITE/OUT "{cmd}: Copying {original}{ext} to {frame}{ext}"
  $cp {original}{ext} {frame}{ext}
 ENDIF
!
! Get the size in pixels of the CCD
!
 size_ccd(1) = m$value({frame},npix(1)) 
 size_ccd(2) = m$value({frame},npix(2))
!
! If size_area not given, set it to default 15x15
!
  IF p2 .eq. "0" THEN
    size_area(1) = 15
    size_area(2) = 15
  ENDIF
!
! First reset display window
!
 CLEAR/DISP >NULL
!
! Load the image
!
 WRITE/OUT "{cmd}: Loading image {frame}"
 LOAD/IMA {frame} >NULL
!
! Ask if the input file (original) has already bias & dark subtracted
!
 askContinue:
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "Is {original} with bias and dark subtracted [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO continue
 ELSEIF reply(:1) .eq. "n" THEN
   WRITE/OUT "{cmd}: Removing bias & dark as a constant"
   !
   ! Calculate (aprox) the size for the bias-dark area, and set the cursor to
   ! square with that size.
   !
   dark_area(1) = ({size_ccd(1)} - 20)
   dark_area(2) = ({size_ccd(2)}/2 - 50)
   coord(1) = {dark_area(1)} + 10
   coord(2) = {dark_area(2)} + 10
   SET/CURSOR 2 rect 10,10,{coord(1)},{coord(2)}
   !
   ! Compute bias+dark constant (as median of the dark_area)
   !
   WRITE/OUT "{cmd}: Move cursor in the display area and a rectangular will pop-up"
   WRITE/OUT "{cmd}: The rectangular area will be used to calculate the "
   WRITE/OUT "{cmd}: bias+dark value (median)."
   WRITE/OUT "{cmd}: Resize the rectangular with arrows if necessary."
   WRITE/OUT "{cmd}: Click with left mouse button to set area (e.g. above spectrum)."
   WRITE/OUT "{cmd}: Click with middle mouse button to get median value."
   STAT/IMAGE ? cursor >NULL
   stat_dark = OUTPUTR(8)
   WRITE/OUT ""
   WRITE/OUT "{cmd}: Median in selected area = {stat_dark}"
   !
   ! Compute a temporary image with the bias+dark subtracted
   !
   COMPUTE/IMA {frame}bd = {frame} - {stat_dark}
   frame = frame + "bd"
   WRITE/OUT ""
   WRITE/OUT "{cmd}: Creating {frame} with bias+dark subtracted"
   !
   ! Remove previous ovelays (crosses and other marks)
   !
   CLEAR/CHAN ov
   !
   !
   ! Loading image without bias+dark
   !
   WRITE/OUT "{cmd}: Loading image {frame}"
   LOAD/IMA {frame} >NULL
 ELSE
   GOTO askContinue
 ENDIF
 continue:
!
! Set cursor with the size to compute total intensity
!
 coord(1) = {size_area(1)} + 10
 coord(2) = {size_area(2)} + 10
 SET/CURSOR 2 rect 10,10,{coord(1)},{coord(2)}
!
! 
 WRITE/OUT 
 WRITE/OUT "{cmd}: Move cursor in the display area where it will transform on"
 WRITE/OUT "{cmd}: and a square of {size_area} pixels pop-ups"
 WRITE/OUT "{cmd}: Move the square on top of an emmission line where you want to"
 WRITE/OUT "{cmd}: calculate statistics"
 STAT/IMAGE ? cursor 
!
! Calculate FWHM 
!
 WRITE/OUT ""
 WRITE/OUT "{cmd}: Move no the square on top of an emmission line where you want to"
 WRITE/OUT "{cmd}: calculate FWHM"
 CENTER/GAUSS 
!
! Clean temporary files
!
 WRITE/OUT "{cmd}: cleaning temporary files and finishing"
 $rm -f NULL
 $rm -f {original}_*
!
 ECHO/OFF
