! $Id: extracting.prg,v 1.3 1999/08/23 21:16:10 cguirao Exp $
! $Name: FIASCO-1_3 $ 
!
! AUTHOR: C. Guirao
!
! NAME: extracting.prg
!
! INPUT: 
!     p1 REQUIRED: BDF input image of spectrum to be reduced
!     p2 OPTIONAL: BDF output as one dimensional image
!     p3 OPTIONAL: graph/nograph (default: nograph)
!     p4 OPTIONAL: Order and niter for command EXTRACT/LONG
!     p5 OPTIONAL: Read-out-noise (ADU),inverse gain factor and threshold 
!                  for the removal of cosmic ray hits
!
! OUTPUT: p2 
!
! RETURN: q1=ycentrum (row central of spectrum)
!         q2=ystart   (row start to average)
!         q3=yend     (row end to average)
!   
! CONVENTION:
!
! USAGE: @@ extracting inframe.bdf outframe.bdf 
!
!*******************************************************************************
! ECHO/FULL
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ? 	  IMA "Enter BDF image to be binned: "
 DEFINE/PARA p2 + ?      "Enter BDF output frame ? "
 DEFINE/PARA p3 + ?      "Enter graph/nograph option "
 DEFINE/PARA p4 3,3       NUM
 DEFINE/PARA p5 7,2,3     NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20  	"extracting"
 DEFINE/LOCA inframe/c/1/40	{p1}
 DEFINE/LOCA outframe/c/1/40	{p2}
 DEFINE/LOCA mode/c/1/10	{p3}
 DEFINE/LOCA dark_table/c/1/40 "master_dark.tbl"
 DEFINE/LOCA bias/c/1/40        "master_bias.bdf"
!
 DEFINE/LOCA ystart/i/1/1       0
 DEFINE/LOCA yend/i/1/1         0
 DEFINE/LOCA reply/c/1/1        y
 DEFINE/LOCA indx/i/1/1         0
 DEFINE/LOCA ycentral/r/1/1	0
 DEFINE/LOCA ycentrum/i/1/1	0
 DEFINE/LOCA yfwhm/r/1/1	0.0
 DEFINE/LOCA frame/c/1/40    
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10         "" ? +lower_levels
 DEFINE/LOCA next/i/1/1         0
 DEFINE/LOCA yp/i/1/1           18
!
! Check if plot or not 
!
 IF p3 .eq. "+" THEN
   mode = "graph"
 ELSE 
   IF mode .ne. "graph" THEN
     IF mode .ne. "nograph" THEN
       WRITE/OUT "{cmd}: {p3} is a wrong parameter. [graph/nograph]"
       RETURN/EXIT
     ENDIF
   ENDIF
 ENDIF

!
! Gets basename and extension of input file
!
 @@ file_ext {inframe}
 frame = basename
!
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
   @@ compute_fits {inframe} {frame}
   !
   ! Bias subtraction
   !
   IF m$exist(bias) .eq. 1 THEN
     COMPUTE/IMA {frame}_b = {frame} - {bias}
     RENAME {frame}_b {frame}
     WRITE/OUT "{cmd}: creating {frame} with {bias} subtracted"
   ELSE
     WRITE/OUT "{cmd}: {bias} not found. No bias subtracted"
   ENDIF
   !
   ! Dark current subtraction
   !
   IF m$exist(dark_table) .eq. 1 THEN
     @@ dark_subtraction {frame} {frame}_d
     RENAME {frame}_d {frame}
     WRITE/OUT "{cmd}: creating {frame} with dark_current subtracted"
   ELSE
     WRITE/OUT "{cmd}: {dark_table} not found. No dark_current subtracted"
   ENDIF
 ENDIF
!
! If BDF frame name {p2} is missing, use input frame without extension.
!
!  Strip the extension to input file (anything after ".")
!
 IF p2 .eq. "+" THEN
   outframe =  basename + "_1"
 ENDIF
!
! Initiate contex LONG and create a grap
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT long >NULL
 ENDIF
 IF mode .eq. "graph" THEN
   SET/GRAP DEFAULT=
   SET/GRAP pm=1 fount=1 xa=auto ya=auto
   DELETE/GRAP 0 >NULL
   GRAPH/SPEC ? 1,20 0
 ENDIF
!
! Search for row central containing the spectrum
!
 @@ search {frame}
!
 ycentral = m$value(gauss,:XCEN,@1)
 ycentrum = ycentral + 0.5
 yfwhm = m$value(gauss,:XFWHM,@1)
 ystart = {ycentral} - ({yfwhm} * 0.75) + 0.5
 yend =   {ycentral} + ({yfwhm} * 0.75) + 0.5
 WRITE/OUT "{cmd}: spectrum located in rows {ystart},{yend}"
!
! Plotting central row of original frame
!
 WRITE/OUT "{cmd}: ploting central row {ycentrum} from {frame}"
 IF mode .eq. "graph" THEN
   SET/GRAP colour=1
   PLOT/ROW {frame} {ycentrum} >NULL
   LABEL/GRAPH "{next}-> central row {ycentrum} of the raw frame" 10,{yp},mm 0 0.75 1
   next = next + 1
   yp = yp - 3
 ENDIF
!
! Extract spectrum from the CCD frame with AVERAGE/ROW. The rows of the
! spectrum are added together.
!
 AVERAGE/ROW {outframe} = {frame} {ystart},{yend} 
 WRITE/OUT "{cmd}: averaging and ploting {outframe} from {ystart} to {yend}"
 IF mode .eq. "graph" THEN
   SET/GRAP colour=2
   OVERPLOT/ROW {outframe} >NULL
   LABEL/GRAPH "{next}-> Average of rows {ystart}-{yend} in raw frame" 10,{yp},mm 0 0.75 1
   next = next + 1
   yp = yp - 3
 ENDIF
!
! Extract spectrum from the CCD frame with EXTRACT/LONG. The rows of the 
! spectrum are added with weights which are chosen for optimal S/N-ratio of
! the resulting spectrum. Cosmic ray hits are removed by analyzing the 
! profile perpendicular to the dispersion (assumed to be along the x-axis).
!
! write/out EXTRACT/LONG {frame} {outframe} ? {ystart},{yend}  {p4} {p5} 
! EXTRACT/LONG {frame} {outframe} ? {ystart},{yend}  {p4} {p5} >NULL
! WRITE/OUT "{cmd}: averaging and ploting {frame} from {ystart} to {yend}"
! IF mode .eq. "graph" THEN
!   SET/GRAPH colour=2
!   OVERPLOT/ROW {outframe} >NULL
!   LABEL/GRAPH "{next}-> Average of all rows with spectrum" 10,{yp},mm 0 0.75 1
!   next = next + 1
!   yp = yp - 3
!   SET/GRAPH colour=1
! ENDIF
!
! Clean all temporary files.
!
 WRITE/OUT "{cmd}: cleaning temporary files and finishing"
 IF mode .eq. "graph" THEN
   $rm -f graph_wn*.plt
 ENDIF
 $rm -f cosmics.bdf middumm*.bdf
 $rm -f coerbr.tbl middumm*.tbl
 $rm -f gauss.tbl
! $rm -f line.tbl
 $rm -f NULL
!
 ECHO/OFF
!
! return the ycentrum, ystart and yend 
!
 RETURN {ycentrum} {ystart} {yend}
