! $Id: flux_calib.prg,v 1.4 2000/07/13 21:38:10 cguirao Exp $
! $Name: FIASCO-1_3 $ 
!
! AUTHOR: J.M Alcala & C. Guirao 
!  
! NAME: flux_calib.prg 
!   Relative flux calibration MIDAS procedure using a flux table of 
!   a standard star and the 1-dim wave-calibrated spectrum of the
!   same observed standard star (reduced previously with pipeline.prg)
!
! INPUT: FITS image with the spectrum of a calibration star (default Vega)
!        or the 1dim spectra of a calibration star already calibrated in
!        wavelength.
!
! OUTPUT: A BDF function: master_flux resulted from dividing the real
!         spectrum of the calibration star with the observed one.
!
! USAGE: @@ flux_calib <calibration_star>.bdf <calibration_star>.tbl
!
!*******************************************************************************
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: MIDAS image with the spectrum of a calibration star OR
!              1-dim image (.bdf) of an already wavelength calibration star
! p2 OPTIONAL: TABLE with flux values of the calibration star.
!
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ? 	  	IMA "Enter image to be calibrated: "
 DEFINE/PARA p2 "hr7001.tbl"	TAB
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20         "flux_calib"
 DEFINE/LOCA fitsframe/c/1/40   {p1}
 DEFINE/LOCA std_table/c/1/40   {p2}
 DEFINE/LOCA frame/c/1/40
 DEFINE/LOCA table/c/1/40
 DEFINE/LOCA flux/c/1/40        "master_flux.bdf"
!
! Do not remove +lower_levels for 'y', SEARCH/LONG does not work without it
!
 DEFINE/LOCA reply/c/1/1        y
 DEFINE/LOCA naxis/i/1/1
 DEFINE/LOCA stat_flux/r/1/1	0.0
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10         "" ? +lower_levels
!
 IF m$exist(flux) .eq. 1 THEN
   WRITE/OUT
   WRITE/OUT "WARNING: A CALIBRATION FLUX IMAGE {flux} ALREADY EXIST"
   WRITE/OUT "         REMOVE IT BEFORE CONTINUING"
   WRITE/KEYW reply/c/1/1 y
   INQUIRE/KEYW reply "Remove {flux} file [yn] (y)?"
   IF reply(:1) .ne. "y" THEN
     ECHO/OFF
     RETURN/EXIT
   ENDIF
   DELETE/IMAG {flux} NO
 ENDIF
!
 IF m$exist(std_table) .eq. 0 THEN
   WRITE/OUT
   WRITE/OUT "{cmd}: ERROR. Table file: {std_table} for calibration star not found"
   WRITE/OUT "       Tables for most common calibration stars are available in"
   WRITE/OUT "       $MID_STANDARD"
   RETURN/EXIT
 ENDIF
!
! Strip the TABL extension from the table input filename
!
 @@ file_ext {std_table}
 table = basename
!
! Strip the FITS or BDF extension from the frame input filename
!
 @@ file_ext {fitsframe}
 frame = basename
!
! If input file is FITS first reduce it to 1-dim
!
 IF ext .ne. ".bdf" THEN
   WRITE/OUT "{cmd}: REDUCING {fitsframe} TO 1-DIM WAVE-CALIBRATED IMAGE"
   @@ pipeline {fitsframe} 
   INDISK/FITS {frame}.mt {frame} >NULL
 ENDIF
!
! Initiate contex LONG
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT long >NULL
 ENDIF
!
! Relative flux calibration by dividing original with response function.
!
 WRITE/OUT "{cmd}: converting table to image in the same wavelength range"
 CONVERT/TABLE i{table} = {table} #1 #2 {frame} spline
 WRITE/OUT "{cmd}: generate flux function by dividing table-image with observed one"
 COMPUTE/IMAGE resp = {frame} / i{table}
!
! Create a graphical window and plot the flux to be normalized
!
 DELETE/GRAP >NULL
 SET/GRAPH pm=1 fount=1 xa=auto ya=auto
 GRAPH/SPEC
 PLOT resp
!
 WRITE/OUT "{cmd}: fitting the response function"
 WRITE/OUT "{cmd}: use your right mouse button to enter coordinates"
!
 repeat_normal: 
 NORMAL/SPEC resp nresp 
 PLOT nresp
!
 repeat_question1:
 WRITE/KEYW reply/c/1/1 n
 INQUIRE/KEYW reply "Adding more points to the fitting [yn] (n)?"
 IF reply(:1) .eq. "y" THEN
   NORMAL/SPEC nresp nnresp add
   RENAME nnresp nresp
   PLOT nresp
 ELSEIF reply(:1) .ne. "n" THEN
   GOTO repeat_question1
 ENDIF
!
 repeat_question2: 
 WRITE/KEYW reply/c/1/1 n
 INQUIRE/KEYW reply "Repeat the fitting [yn] (n)?"
 IF reply(:1) .eq. "y" THEN
   GOTO repeat_normal
 ELSEIF reply(:1) .ne. "n" THEN
   GOTO repeat_question2
 ENDIF
!
! Normalize function to 1
!
 WRITE/OUT "{cmd}: normalizing the response function to 1"
 STATIS/IMA nresp >NULL
 stat_flux = OUTPUTR(8)
 COMPUTE/IMA {flux} = nresp / {stat_flux}
 PLOT {flux}
!
 WRITE/OUT "{cmd}: flux coeficient function located in {flux}"
!
! Clean all temporary files.
!
 $rm -f resp.bdf nresp.bdf
 $rm -f i{table}.bdf
 $rm -f FIT1D.tbl
 $rm -f NULL
!
 ECHO/OFF
