! $Id: master_dark.prg,v 1.7 2001/08/20 15:41:51 cguirao Exp $
! $Name: FIASCO-1_3 $ 
!
! AUTHOR: C.Guirao
!
! NAME: master_dark.prg
!   Data reduction procedure to create a catalog and a table with exposure
!   values 
!
! INPUT: 
!   - Set of master dark_current images (bias subtracted) separated with 
!     commas, wildcards also allowed.
!
! OUTPUT: 
!   - master_dark.cat: A catalog of dark current frame at different exposures
!   - master_dark.tbl: A table with the exposure values of dark_current
!                      frames in the catalog.
!
!**********************************************************************       
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: BDF images of master dark_currents at different exposures
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ?         IMA "Enter BDF images of master dark_currents: "
 DEFINE/PARA p2 0 ?       NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20		"master_dark"
 DEFINE/LOCA frame/c/1/40	{p1}
 DEFINE/LOCA area/i/1/4		{p2}   
 DEFINE/LOCA dark_frame/c/1/40
 DEFINE/LOCA exposure/i/1/1 	0
 DEFINE/LOCA linpair/i/1/1  	0   ? +lower
 DEFINE/LOCA catal/i/1/1 	0
!
! Get the name of first image 
!
 $echo {p1} | gawk '{print $1}' >out1
 $cat out1 | gawk -F, '{print $1}' >out2
 WRITE/KEYW frame <out2
 $rm -f out1 out2
!
! Get the size in pixels of the CCD
!
! size_ccd(1) = m$value({frame},npix(1))
! size_ccd(2) = m$value({frame},npix(2))
!!
!IF p2 .eq. "0" THEN
!  area(1) = 10
!  area(2) = 10
!  area(3) = size_ccd(1) - 10
!  area(4) = 110
!ENDIF
!
! Remove previous table and catalog files
!
 $rm -f master_dark.cat
 $rm -f master_dark.tbl
!
! Create a catalog where to save BDF master dark_currents
!
 CREATE/ICAT master_dark.cat {p1}
!
! Create a table where to save exposure times of master dark_currents
!
 CREATE/TAB master_dark
 CREATE/COL master_dark :FRAME C*20
 CREATE/COL master_dark :EXPOSURE
!
! For each master dark_current:
!
 linpair = 1
 LOOP:
!
 STORE/FRAM in_b master_dark.cat 1 FINITO
!
   WRITE/KEY dark_frame {in_b} 
!
   exposure = m$value({dark_frame},O_TIME(7))
   WRITE/TABL master_dark :FRAME {linpair} {dark_frame}
   WRITE/TABL master_dark :EXPOSURE {linpair} {exposure} 
   linpair = linpair + 1
   WRITE/OUT "{cmd}: Exposure time for {dark_frame}= {exposure}"
!
 GOTO LOOP
 FINITO:
!
! Sort the table in ascending EXPOSURE values
!
 SORT/TAB master_dark.tbl :EXPOSURE
!
! Clean all temporary files.
!
 WRITE/OUT "{cmd}: cleaning temporary files and finishing"
 $rm -f NULL
 $rm -f dirfile.ascii
!
 ECHO/OFF
