! $Id: master_trans.prg,v 1.1 1999/07/24 14:45:30 cguirao Exp $
! $Name: FIASCO-1_3 $ 
!
! AUTHOR: C.Guirao
!
! NAME: master_trans.prg
!   Data reduction procedure to create a "master_trans.bdf" a function 
!   representing the low frequency modulation of the system: CCD, spectograph
!   fiber and telescope.
!   To calculate it we take dome flat fields, we average them
!   and remove the pixel to pixel variations. We then rebin the central
!   row +/- FWHM number of pixels
!
! INPUT: 
!   - P1: Set of dome flat field fits frames 
!   - P2: Blackbody temperature of the flatfield lamp in Kelvin
!
! OUTPUT: 
!   - P3: master master_trans BDF image
!
!**********************************************************************       
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS images from dome flat fields
! p2 OPTIONAL: Blackbody temperature for flat field lamp
! p3 OPTIONAL: Name of the master modulation file to be generated
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ?         	 IMA "Enter FITS frames of dome flatfields: "
 DEFINE/PARA p2 3200      	 NUM 
 DEFINE/PARA p3 master_trans.bdf IMA
!
! Convert paramaters into variables:
!
 DEFINE/LOCA temperature/i/1/1		{p2}		
 DEFINE/LOCA output_frame/c/1/40   	{p3}
 DEFINE/LOCA flat/c/1/40		"master_flat.bdf"
 DEFINE/LOCA session/c/1/40		"session.tbl"
 DEFINE/LOCA frame/c/1/40		"trans_a"
 DEFINE/LOCA original/c/1/40
 DEFINE/LOCA extension/c/1/4		
!
! Do not remove +lower_levels for 'y', SEARCH/LONG does not work without it
!
 DEFINE/LOCA y/r/1/1            0.0  ? +lower_levels
 DEFINE/LOCA size_ccd/i/1/2
 DEFINE/LOCA ystart/i/1/1       0
 DEFINE/LOCA yend/i/1/1         0
 DEFINE/LOCA reply/c/1/1        y
 DEFINE/LOCA indx/i/1/1         0
 DEFINE/LOCA end_indx/i/1/1     0
 DEFINE/LOCA ycentral/r/1/1     0
 DEFINE/LOCA ycentrum/i/1/1     0
 DEFINE/LOCA yfwhm/r/1/1        0.0
 DEFINE/LOCA average/r/1/1      0.0
!
! Strip the FITS or BDF extension of the frame output filename
!
 original = frame
 indx = m$index(output_frame,".")
 IF indx .ne. 0 THEN
   end_indx = {indx} + 5
   extension = output_frame({indx}:{end_indx})
   IF extension .ne. ".bdf" THEN
     WRITE/OUT "ERROR: FILE {output_frame} MISSES EXTENSION .bdf"
     RETURN/EXIT
   ENDIF
 ENDIF
!
! Initiate contex LONG and create a grap
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT long >NULL
 ENDIF
 RESET/DISP
 SET/GRAP pm=1 fount=1 xa=auto ya=auto
 DELETE/GRAP 1 >NULL
 GRAPH/SPEC ? 1,20 1
!
! average the input files (bias is also substracted here)
! dark current is neglictable in sort exposures (<1min)
!
 WRITE/OUT "master_trans: averaging {p1} into {frame}"
 @@ averaging {p1} {frame}
!
! Get the size in pixels of the CCD
!
 size_ccd(1) = m$value({frame},NPIX(1))
 size_ccd(2) = m$value({frame},NPIX(2))
 WRITE/OUT "master_trans: {frame} CCD size {size_ccd(1)}x{size_ccd(2)}"
!
! Search for row central containing the spectrum
!
 @@ search {frame}
!
 ycentral = m$value(gauss,:XCEN,@1)
 ycentrum = ycentral + 0.5
 yfwhm = m$value(gauss,:XFWHM,@1)
 ystart = {ycentral} - ({yfwhm} * 0.75) + 0.5
 yend =   {ycentral} + ({yfwhm} * 0.75) + 0.5
! ystart = {ycentral} - ({yfwhm} * 0.5) + 0.5
! yend =   {ycentral} + ({yfwhm} * 0.5) + 0.5
 WRITE/OUT "master_trans: spectrum located in central row {ycentral} with FWHM {yfwhm}"
 WRITE/OUT "master_trans: spectrum located in rows {ystart},{yend}"
!
! Plotting central row of original average_flat
!
 WRITE/OUT "master_trans: ploting central row {ycentrum} from {frame}"
 SET/GRAP colour=1
 PLOT/ROW {frame} {ycentrum}
 LABEL/GRAPH "1-> central row of the raw spectrum" 10,12,mm 0 0.75 1
!
! Flat-fielding (removing pixel to pixel variations)
!
 IF m$exist(flat) .eq. 1 THEN
   COMPUTE/IMA {frame}f = {frame} / {flat} >NULL
   frame = frame + "f"
   WRITE/OUT "master_trans: creating {frame} with {flat} divided"
   !
   ! Overploting central row of original frame - bias - dark - flat
   !
   WRITE/OUT "master_trans: overploting central row {ycentrum} from {frame}"
   SET/GRAP colour=2
   OVERPLOT/ROW {frame} {ycentrum}
   LABEL/GRAPH "2-> with flat-field divided" 10,9,mm 0 0.75 1
 ELSE
   askToContinue1:
   WRITE/KEYW reply/c/1/1 y
   WRITE/OUT "master_trans: {flat} not found"
   INQUIRE/KEYW reply "Do you want to continue [yn] (y)?"
   IF reply(:1) .eq. "y" THEN
     GOTO continue1
   ELSEIF reply(:1) .eq. "n" THEN
     ECHO/OFF
     RETURN/EXIT
   ELSE
     GOTO askToContinue1
   ENDIF
 ENDIF
 continue1:
!
! Extract spectrum from the CCD frame with AVERAGE/ROW. The rows of the
! spectrum are averaged.
!
! WRITE/OUT "master_trans: ironing and ploting {frame} from {ystart} to {yend}"
! AVERAGE/ROW {frame}1 = {frame} {ystart},{yend}  sum
! AVERAGE/ROW {frame}1 = {frame} {ystart},{yend}
! frame = frame + "1"
! 
!
! Extract spectrum from the CCD frame with FILTER/MEDI. 
! Using FILTER/MEDI gets smoother result. 
!
 WRITE/OUT "master_trans: ironing {frame} with FILTER/MEDI"
! FILTER/MEDI {frame} {frame}m 5,{yfwhm},0 >NULL
 FILTER/MEDI {frame} {frame}m 5,3,0 >NULL
 frame = frame + "m"
 EXTRAC/IMAG {frame}1 = {frame} [@1,@{ycentral}:@{size_ccd(1)},@{ycentral}]
 frame = frame + "1"
!
 SET/GRAPH colour=3
 OVERPLOT/ROW {frame}
 LABEL/GRAPH "3-> Average of all rows in CCD with spectrum" 10,6,mm 0 0.75 1
!
! Wavelength calibration
!
 IF m$exist(session) .eq. 1 THEN
   INIT/LONG {session} >NULL
   REBIN/LONG {frame} {frame}w >NULL
   COPY/DD {frame} *,3 {frame}w
   frame = frame + "w"
   WRITE/OUT "master_trans: overploting {frame} calibrated in wavelength"
   DELETE/GRAP 2 >NULL
   GRAPH/SPEC ? 20,40 2
   SET/GRAPH colour=1
   PLOT/ROW {frame}
   LABEL/GRAPH "4-> calibrated in wavelength" 10,9,mm 0 0.75 1
  ELSE
   WRITE/OUT "master_trans: ERROR: File {session} NOT FOUND"
   WRITE/OUT "master_trans: USAGE: @@ wave_calib <calibration_lamp>.fit"
   RETURN/EXIT
 ENDIF
!
! Correct flat field with the curve of a blackbody at {temperature} K 
! 1- Create a blackbody_{temperature} table
! 2- Convert table to image using {frame} as a reference
! 3- Divide {frame} with blackbody response 
!
 WRITE/OUT "master_trans: creating blackbody table at {temperature} K"
 @@ blackbody {temperature}
 $ sleep 3
 WRITE/OUT "master_trans: creating blackbody image at {temperature} K"
 CONVERT/TABLE iblackbody_{temperature} = blackbody_{temperature} #1 #2 {frame} spline
 WRITE/OUT "master_trans: correcting transmission with blackbody image"
 COMPUTE/IMAGE {frame}b = {frame} / iblackbody_{temperature}
 frame = frame + "b"
 WRITE/OUT "master_trans: overploting {frame} calibrated in wavelength"
!
! Normalize to 1 by dividing by the average value
!
 WRITE/OUT "master_trans: normalizing with average from STATIS/IMAG"
 STATIS/IMAG {frame} >NULL
 average = OUTPUTR(8)
 COMPUT/IMAG  {frame}n = ({frame} / {average}) >NULL
 frame = frame + "n"
!
! Rename result frame as output_frame
!
 WRITE/OUT "master_trans: {output_frame} created"
 RENAME/IMA {frame} {output_frame}
!
! Final plot
!
 DELETE/GRAP 0 > NULL
 SET/GRAPH colour=1
 GRAPH/SPEC ? 40,60 0
 PLOT {output_frame}
 LABEL/GRAPH "5-> adjusted with blackbody at {temperature} K" 10,6,mm 0 0.75 1
!
! Clean all temporary files.
!
 WRITE/OUT "master_trans: cleaning temporary files and finishing."
 $rm -f *blackbody_{temperature}.*
 $rm -f {original}*.bdf
 $rm -f graph_wn*.plt
 $rm -f coerbr.tbl middumm*.tbl
 $rm -f gauss.tbl
 $rm -f line.tbl
 $rm -f midlis*.prg
 $rm -f lntab.tbl
 $rm -f postscript.ps
 $rm -f NULL
!
 ECHO/OFF
