! $Id: pipeline.prg,v 1.11 2000/07/13 21:40:50 cguirao Exp $
! $Name: FIASCO-1_3 $ 
!
! AUTHOR: J.M Alcala & C. Guirao
!
! NAME: pipeline.prg
!   Standard reduction MIDAS procedure for spectra located in  rows in a 
!   FITS file. It includes bias and dark correction,  flatfield division 
!   averaged, and calibration in wavelength and flux.  
!
! INPUT: 
!   CCD image in FITS format with spectrum in rows (e.g. sirius.fit)
!
! OUTPUT: 
!   One-dimensional image (e.g. sirius_bdf1tws.bdf) alredy reduced
!   
! CONVENTION:
!   b with bias subtraction
!   d with dark-current subtraction
!   f with flat-field division
!   1 averaged to one-dimensional image
!   w calibrated in wavelength
!   t calibrated with transmission (low frecuency)
!   s calibrated in flux with a standard star
!
! USAGE: @@ pipeline frame.fit
!
!*******************************************************************************
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS image of spectrum to be reduced
! p2 OPTIONAL: Threshold for SET/LONG
! p2 OPTIONAL: Order and niter for command EXTRACT/LONG
! p3 OPTIONAL: Read-out-noise (ADU),inverse gain factor and threshold for the
!              removal of cosmic ray hits
!
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ? 	  IMA "Enter FITS image to be calibrated: "
 DEFINE/PARA p2 1000      NUM
 DEFINE/PARA p3 3,3       NUM
 DEFINE/PARA p4 7,2,3     NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20		"pipeline"
 DEFINE/LOCA fitsframe/c/1/40   {p1}
 DEFINE/LOCA threshold/i/1/1    {p2}
 DEFINE/LOCA order_nitter/i/1/2 {p3}
 DEFINE/LOCA ron_g_sigma/i/1/3  {p4}
 DEFINE/LOCA original/c/1/40
 DEFINE/LOCA frame/c/1/40
 DEFINE/LOCA bias/c/1/40         "master_bias.bdf"
 DEFINE/LOCA dark_table/c/1/40   "master_dark.tbl"
 DEFINE/LOCA flat/c/1/40         "master_flat.bdf"
 DEFINE/LOCA flat_extracted/c/1/40 "flat_extracted.bdf"
 DEFINE/LOCA transmission/c/1/40 "master_trans.bdf"
 DEFINE/LOCA session/c/1/40      "session.tbl"
 DEFINE/LOCA flux/c/1/40         "master_flux.bdf"
 DEFINE/LOCA printer/c/1/40      "postscript"
 DEFINE/LOCA size_ccd/i/1/2
!
! DEFINE/LOCA datum/c/1/40       "Date: 24/03/99"
! DEFINE/LOCA telescope/c/1/40   "Meade/LX200 10in"
! DEFINE/LOCA fiber/c/1/40       "Fiber: 200um"
! DEFINE/LOCA grating/c/1/40     "Grating: 600c/mm"
! DEFINE/LOCA objective/c/1/40   "Objective: 100mm"
! DEFINE/LOCA camera/c/1/40      "CCD: Hale512"
!
 DEFINE/LOCA ystart/i/1/1       0
 DEFINE/LOCA yend/i/1/1         0
 DEFINE/LOCA reply/c/1/1        y
 DEFINE/LOCA ycentral/r/1/1	0
 DEFINE/LOCA ycentrum/i/1/1	0
 DEFINE/LOCA fcentral/r/1/1	0
 DEFINE/LOCA fcentrum/i/1/1	0
 DEFINE/LOCA yfwhm/r/1/1	0.0
 DEFINE/LOCA exposure/i/1/1	0
 DEFINE/LOCA stat_flux/r/1/1    0.0
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10         "" ? +lower_levels
 DEFINE/LOCA next/i/1/1		1
 DEFINE/LOCA colour/i/1/1	1
 DEFINE/LOCA yp/i/1/1		18
!
! WRITE/OUT "{cmd}: ORDER={order_nitter(1)}"
! WRITE/OUT "{cmd}: NITTER={order_nitter(2)}"
! WRITE/OUT "{cmd}: RON={ron_g_sigma(1)}"
! WRITE/OUT "{cmd}: GAIN={ron_g_sigma(2)}"
! WRITE/OUT "{cmd}: SIGMA={ron_g_sigma(3)}"
 WRITE/OUT "{cmd}: VERSION: $Name: FIASCO-1_3 $ "
 WRITE/OUT "{cmd}: BIAS={bias}"
 WRITE/OUT "{cmd}: DARK_TABLE={dark_table}"
 WRITE/OUT "{cmd}: FLAT={flat}"
! WRITE/OUT "{cmd}: TRANSMISSION={transmission}"
 WRITE/OUT "{cmd}: WAVELENGTH CALIBRATION={session}"
 WRITE/OUT "{cmd}: FLUX CALIBRATION={flux}"
 WRITE/OUT 
!
! Gets basename and extension of input file
!
 @@ file_ext {fitsframe}
 original = basename
 frame = original + "_"
!
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
   @@ compute_fits {fitsframe} {frame}
 ELSE
  $cp {original}{ext} {frame}{ext}
 ENDIF
!
! Initiate contex LONG and create a grap
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT long >NULL
 ENDIF
 RESET/DISP >NULL
 SET/GRAP DEFAULT=
 SET/GRAP pm=1 font=1 xa=auto ya=auto
 DELETE/GRAP 0 >NULL
 GRAPH/SPEC ? 1,20 0
!
! Get the size in pixels of the CCD
!
 size_ccd(1) = m$value({frame},NPIX(1)) 
 size_ccd(2) = m$value({frame},NPIX(2))
 WRITE/OUT "{cmd}: {frame} CCD size {size_ccd(1)}x{size_ccd(2)}"
!
! Get the exposure time
!
 exposure = m$value({frame},O_TIME(7)) 
 WRITE/OUT "{cmd}: exposure time for {frame}: {exposure}"
!
! Search for row central containing the spectrum
!
 @@ search {frame}
!
 ycentral = m$value(gauss,:XCEN,@1)
 ycentrum = ycentral + 0.5
 yfwhm = m$value(gauss,:XFWHM,@1)
 ystart = {ycentral} - ({yfwhm} * 0.75) + 0.5
 yend =   {ycentral} + ({yfwhm} * 0.75) + 0.5
! ystart = {ycentral} - ({yfwhm} * 0.5) + 0.5
! yend =   {ycentral} + ({yfwhm} * 0.5) + 0.5
 WRITE/OUT "{cmd}: spectrum located in rows {ystart},{yend}"
!
! Plotting central row of original frame
!
 WRITE/OUT "{cmd}: ploting central row {ycentrum} from {frame}"
 SET/GRAP colour={colour}
 PLOT/ROW {frame} {ycentrum}
 LABEL/GRAPH "{next}-> central row {ycentrum} of the raw frame" 10,{yp},mm 0 0.75 1
 next = next + 1
 colour = colour + 1
 yp = yp - 3
!
! Bias subtraction
!
 IF m$exist(bias) .eq. 1 THEN 
   COMPUTE/IMA {frame}b = {frame} - {bias}
   frame = frame + "b"
   WRITE/OUT "{cmd}: creating {frame} with {bias} subtracted"
   !
   ! Overploting central row of original frame - bias
   !
   WRITE/OUT "{cmd}: overploting central row {ycentrum} from {frame}"
   SET/GRAP colour={colour}
   OVERPLOT/ROW {frame} {ycentrum} >NULL
   LABEL/GRAPH "{next}-> with bias subtracted" 10,{yp},mm 0 0.75 1
   next = next + 1
   colour = colour + 1
   yp = yp - 3
 ELSE
   WRITE/OUT "{cmd}: {bias} not found. No bias subtracted"
 ENDIF
!
! Dark subtraction
!
 IF m$exist(dark_table) .eq. 1 THEN 
   @@ dark_subtraction {frame} {frame}d
   frame = frame + "d"
   WRITE/OUT "{cmd}: creating {frame} with dark_current subtracted"
   !
   ! Overploting central row of original frame - bias - dark
   !
   WRITE/OUT "{cmd}: overploting central row {ycentrum} from {frame}"
   SET/GRAP colour={colour}
   OVERPLOT/ROW {frame} {ycentrum} >NULL
   LABEL/GRAPH "{next}-> with dark-current subtracted" 10,{yp},mm 0 0.75 1
   next = next + 1
   colour = colour + 1
   yp = yp - 3
 ELSE
   WRITE/OUT "{cmd}: {dark_table} not found. No dark_current subtracted"
 ENDIF
!
! Extract spectrum from the CCD frame. The rows of the
! spectrum are added together (binning) in one-dimensional image.
! Results are returned in q1, q2 and q3
!
 @@ extracting {frame} {frame}1 nograph
 frame = frame + "1"
 DEFINE/LOCA centrum/i/1/1 {q1}
 DEFINE/LOCA start/i/1/1   {q2}
 DEFINE/LOCA end/i/1/1     {q3}
 WRITE/OUT "{cmd}: Returned values: {centrum}, {start}, {end}"
 !
 ! Overploting resulted 1-dimensional image
 !
 WRITE/OUT "{cmd}: overploting one-dimensional image"
 SET/GRAPH colour={colour}
 OVERPLOT/ROW {frame} >NULL
 LABEL/GRAPH "{next}-> Average of all rows {start}-{end} in raw frame" 10,{yp},mm 0 0.75 1
 next = next + 1
 colour = colour + 1
 yp = yp - 3
!
! Flat-fielding
!
!
! Search for row central for flat-field and issue a warning if
! if does not match row central for spectra
!
 IF m$exist(flat) .eq. 1 THEN
   $rm -f gauss.tbl
   @@ search {flat}
   fcentral = m$value(gauss,:XCEN,@1)
   fcentrum = fcentral + 0.5
   IF centrum .ne. fcentrum THEN
     askToContinueFlat:
     WRITE/KEYW reply/c/1/1 y
     WRIT/OUT
     WRITE/OUT "{cmd}: WARNING row central in spectrum ({centrum}) differs"
     WRITE/OUT "          differs from row central in flat-field ({fcentrum})"
     INQUIRE/KEYW reply "Do you want to continue [yn] (y)?"
     IF reply(:1) .eq. "y" THEN
       GOTO continueFlat
     ELSEIF reply(:1) .eq. "n" THEN
       ECHO/OFF
       RETURN/EXIT
     ELSE
       GOTO askToContinueFlat
     ENDIF
   ENDIF
   ContinueFlat:
   !
   ! Extract 1dim flat by averaging same rows as for spectrum, then
   ! divide 1dim spectrum with 1dim flat.
   !
   WRITE/OUT "{cmd}: creating {flat} from rows {start} to {end}"
   AVERAGE/ROW {flat_extracted} = {flat} {start},{end} 
   !
   COMPUTE/IMA {frame}f = {frame} / {flat_extracted} >NULL
   frame = frame + "f"
   WRITE/OUT "{cmd}: creating {frame} with {flat_extracted} divided"
   !
   ! Overploting 1dim with flatfield divided
   !
   WRITE/OUT "{cmd}: overploting 1-dim with flat divided"
   SET/GRAP colour={colour}
   OVERPLOT/ROW {frame} >NULL
   LABEL/GRAPH "{next}-> with flat-field divided" 10,{yp},mm 0 0.75 1
   next = next + 1
   colour = colour + 1
   yp = yp - 3
 ELSE
   WRITE/OUT "{cmd}: {flat} not found. No flat divided"
 ENDIF
!
! Wavelength calibration 
!
 IF m$exist(session) .eq. 1 THEN
   INIT/LONG {session} >NULL
   REBIN/LONG {frame} {frame}w >NULL
   COPY/DD {frame} *,3 {frame}w
   frame = frame + "w"
   WRITE/OUT "{cmd}: overploting {frame} calibrated in wavelength"
   DELETE/GRAP 1 >NULL
   GRAPH/SPEC ? 20,40 1
   yp = 18
   colour = 1
   SET/GRAPH colour={colour}
   PLOT/ROW {frame}
   LABEL/GRAPH "{next}-> calibrated in wavelength" 10,{yp},mm 0 0.75 1
   next = next + 1
   colour = colour + 1
   yp = yp - 3
 ELSE
   WRITE/OUT "{cmd}: ERROR: File {session} NOT FOUND"
   WRITE/OUT "{cmd}: USAGE: @@ wave_calib <calibration_lamp>.fit"
   RETURN/EXIT
 ENDIF
!
! Relative flux calibration by dividing original with response function.
! Response function "resp1" obtained with:
! Convert table into image:
!   convert/table i<std_table> = <std_table> #1 #2 <std_obs_spectrum_d1> spline
!   e.g.: convert/table ihr5191 = hr5191 #1 #2 hr5191b_d1 spline
! Obtained temporal response function by dividing the std_observed_spectrum by 
! the previous obtained image of the standard star:
!   compute/image resp = <std_observed_spectrum>_d1 / i<std_table>
!   e.g.: compute/image resp = hr5191b_d1 / ihr5191
! Response function has to be softend by normalyzing
!   normalize/spectrum resp flux
! Any spectrum should be divided by this "flux.bdf" response function.

!
! Transmission (low frecuency variations)
!
! IF m$exist(transmission) .eq. 1 THEN 
!   COMPUTE/IMA {frame}t = {frame} / {transmission} >NULL
!   frame = frame + "t"
!   WRITE/OUT "{cmd}: creating {frame} with {transmission} divided"
!   !
!   ! Overploting resulted 1-dimensional image
!   !
!   WRITE/OUT "{cmd}: overploting calibrated transmission image"
!   SET/GRAP colour={colour}
!   OVERPLOT/ROW {frame} >NULL
!   LABEL/GRAPH "{next}-> with transmission divided" 10,{yp},mm 0 0.75 1
!   next = next + 1
!   colour = colour + 1
!   yp = yp - 3
! ELSE
!   askToContinue4:
!   WRITE/KEYW reply/c/1/1 y
!   WRITE/OUT "{cmd}: {transmission} not found" 
!   INQUIRE/KEYW reply "Do you want to continue [yn] (y)?"
!   IF reply(:1) .eq. "y" THEN
!     GOTO continue4
!   ELSEIF reply(:1) .eq. "n" THEN
!     ECHO/OFF
!     RETURN/EXIT
!   ELSE
!     GOTO askToContinue4
!   ENDIF
! ENDIF
! continue4:

!
! Flux calibration
!
 IF m$exist(flux) .eq. 1 THEN
   COMPUT/IMAG {frame}s = {frame} / {flux} >NULL
   WRITE/OUT "{cmd}: plotting {frame} calibrated in flux"
   frame = frame + "s"
   SET/GRAPH colour={colour}
   OVERPLOT/ROW {frame} >NULL
   LABEL/GRAPH "{next}-> calibrated in relative flux" 10,{yp},mm 0 0.75 1
   next = next + 1
   colour = colour + 1
   yp = yp - 3
 ELSE
   WRITE/OUT "{cmd}: ERROR: File {flux} NOT FOUND"
   WRITE/OUT "{cmd}: USAGE: @@ flux_calib <observed_star>.fit <calibration>.tbl"
   INQUIRE/KEYW reply "{cmd}: Type any key to continue "
 ENDIF
!
! Converting result into FITS
!
 WRITE/OUT "{cmd}: converting {frame}.bdf to {original}.mt"
 OUTDISK/FITS {frame} {original}.mt >NULL
!
! Plot the result in a given wavelength range
!
 DELETE/GRAP 2 >NULL
 SET/GRAPH pm=0 colour=1
 GRAPH/SPEC ? 40,60 2
 PLOT {frame} 
 SET/GRAP colour=2
 OVERPLOT {frame} >NULL
 SET/GRAP colour=1
! OVERPL/LINE 2 6563,0 6563,40e3
! LABEL/GRAPH "H-alpha" 6563,0 0 1.3 1
! LABEL/GRAPH "{original}, {datum}, Exp: {exposure}s." 90,85,mm 0 1 1
! LABEL/GRAPH "{telescope}, {fiber}, {grating}" 90,80,mm 0 0.75 1
! LABEL/GRAPH "{objective}, {camera}" 90,75,mm 0 0.75 1
!
! Send plot to printer and converting to Postscript
!
 COPY/GRAPH {printer}
 WRITE/OUT "{cmd}: converting {frame}.bdf to postscript file {original}.ps"
 $mv {printer}.ps {original}.ps 
!
! askToContinue6:
! WRITE/KEYW reply/c/1/1 y
! INQUIRE/KEYW reply "{cmd}: reseting displays [yn] (y)?"
! IF reply(:1) .eq. "y" THEN
!   GOTO continue6
! ELSEIF reply(:1) .eq. "n" THEN
!   DELETE/GRAP >NULL 
! ELSE
!   GOTO askToContinue6
! ENDIF
! continue6:

!
! Clean intermediate frames.
!
 WRITE/OUT "{cmd}: cleaning intermediate files"
 $rm -f {original}_*.bdf
!
! Clean all temporary files.
!
 WRITE/OUT "{cmd}: cleaning temporary files and finishing"
 $rm -f graph_wn*.plt
 $rm -f cosmics.bdf middumm*.bdf
 $rm -f middumm*.tbl
 $rm -f gauss.tbl
! $rm -f line.tbl
 $rm -f midlis*.prg
 $rm -f lntab.tbl
 $rm -f NULL
 $rm -f bb.bdf
 $rm -f blackbody.bdf
 $rm -f {flat_extracted}
!
 ECHO/OFF
