! $Id: search.prg,v 1.4 2002/01/30 09:11:50 cguirao Exp $
! $Name: FIASCO-1_3 $ 
!
! AUTHOR: C.Guirao
!
! NAME: search.prg 
!   Procedure to detect the central row and fwhm of an image.
!   The input image is assumed to be rotated in such a way that the dispersion
!   direction is oriented along the rows. The wavelength must be increasing 
!   from the left to the right and the STEP descriptors of the image must be
!   positive.
!
! INPUT: 
!   Bi-dimensional FITS or BDF image with an long spectrum oriented in rows.
!   Threshold above background for detecting spectrum central line.
!
! OUTPUT: 
!   A table called gauss.tbl with XCEN indicating the central row for 
!   the expetrum and XFWHM with the fwhm.
!
!******************************************************************************
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS or BDF image
! p2 OPTIONAL: Threshold for SET/LONG 
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ?	IMA "Enter image with expectrum: "
 DEFINE/PARA p2 1000	NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20         "search"
 DEFINE/LOCA frame/c/1/40	{p1}
 DEFINE/LOCA threshold/i/1/1    {p2}
 DEFINE/LOCA size_ccd/r/1/2
 DEFINE/LOCA width/i/1/1   
!
! Do not remove +lower_levels for 'y', SEARCH/LONG does not work without it
!
 DEFINE/LOCA y/r/1/1		0.0  ? +lower_levels
 DEFINE/LOCA start/i/1/1
 DEFINE/LOCA end/i/1/1
 DEFINE/LOCA xstart/r/1/1
 DEFINE/LOCA xend/r/1/1	
 DEFINE/LOCA reply/c/1/1
 DEFINE/LOCA central_y/r/1/1
 DEFINE/LOCA fwhm_y/r/1/1
!
! Get the size in pixels of the CCD, and set start/end for 'x' avoiding borders
!
 size_ccd(1) = m$value({frame},npix(1))
 size_ccd(2) = m$value({frame},npix(2))
 start =  10
 end   = {size_ccd(1)} - 10
!
! Initiate contex LONG
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 0 THEN
   SET/CONTEXT long >NULL
 ENDIF
!
! First we average all the columns except borders. Second,
! search for the maximum 'y' that indicates the row position of the spectrum
! finally extract the row 'y' that contains the spectrum.
!
 AVERAG/COLU out  = {frame} {start},{end} >NULL
repeat:
 SEARCH/LONG out thres={threshold} >NULL
!
! SEARCH/LONG generates a table called line.tbl with as many rows as
! lines are detected. Being an image with only one spectrum, we should
! detect only one line, otherwise we may need to increase the THRESHOLD
! SHOW/TABLE reads the table generated by SEARCH/LONG, the value keyword
! OUTPUTI(2) indicates the number of rows thus the number of lines detected.
! If we detect more than 1 line, the threshold might be too low 
!
 SHOW/TABLE line.tbl >NULL
 IF OUTPUTI(2) .ne. 1 then
   WRITE/KEYW reply/c/1/1 y
   WRITE/OUT
   WRITE/OUT "{cmd}: WARNING: More than one line detected by SEARCH/LONG"
   INQUIRE/KEYW reply "   Doubling current threshold of {threshold} [yn] (y)? "
   IF reply(:1) .eq. "y" THEN
     threshold = 2 * {threshold}
     GOTO repeat
   ELSEIF reply(:1) .eq. "n" THEN
     ECHO/OFF
     RETURN/EXIT
   ENDIF
 ENDIF
!
 COPY/TK line :x  @1 y/r/1/1 
!
! 'y' from COPY/TK contains the central wavelength. Let's create a table
! where we set :XSTART and :XEND to central wavelenght +- 150 pixels. The
! command CENTER/GAUSS will tell us then the FWHM
! 
 xstart = {y} - 150.0
 xend = {y} + 150.0
! 
! Never beyond frame size
!
 if xstart .lt. 0 xstart = 0
 if xend .gt. size_ccd(2) xend = size_ccd(2)
!
 CREATE/TABL gauss
 CREATE/COLU gauss :XSTART
 CREATE/COLU gauss :XEND
 WRITE/TABL gauss :XSTART @1 {xstart}
 WRITE/TABL gauss :XEND   @1 {xend}
 CENTER/GAUS out,gauss gauss >NULL
 central_y = m$value(gauss,:XCEN,@1)
 fwhm_y = m$value(gauss,:XFWHM,@1)
!
 WRITE/OUT "{cmd}: spectrum in {frame} at row: {central_y} with fwhm: {fwhm_y}"
!
! Clean all temporary files
!
 $rm -f NULL
 $rm -f lntab.tbl
! $rm -f line.tbl
 $rm -f out.bdf
!
ECHO/OFF
