! $Id: three_blackbodies.prg,v 1.2 1999/08/14 09:16:43 cguirao Exp $
! $Name: FIASCO-1_3 $
!
! AUTHOR: C. Guirao
!
! NAME: three_blackbodies.prg
!   Procedure that generates a plot with the blackbodies of
!   Arcturus (4700K), Sun (5780K) and Procyon (7000K) in 
!   the wavelength range 300nm - 900nm
!
! INPUT: none
!
! OUTPUT:
!   A postscript file allblackbodies.ps
!
! CONVENTION:
!   Physical Constants (c.g.s units)
!   c -> Speed of light
!   h -> Planck's constant
!   k -> Bolzmann's constant
!
 DEFINE/LOCA c/d/1/1 3.0e10
 DEFINE/LOCA h/d/1/1 6.63e-27
 DEFINE/LOCA k/d/1/1 1.38e-16
!
 DEFINE/LOCA reply/c/1/1  y
 DEFINE/LOCA printer/c/1/40      "postscript"
!
! DEFINE/LOCA t1/d/1/1 3.0e3
 DEFINE/LOCA t2/d/1/1 4.7e3
 DEFINE/LOCA t3/d/1/1 5.780e3
 DEFINE/LOCA t4/d/1/1 7.0e3

!
 CREATE/TABL &t 60 60
 CREATE/COLU &t :Wavelength R*8 E12.6 "nm"
 CREATE/COLU &t :Flux R*8 F12.6 "Bv(T)"
 COMPUT/TABL &t :W = (seq+30)*1e-6
 COMPUT/TABL &t :Wavelength = (seq+30)*10
 COMPUT/TABL &t :F = (2*{h}*{c}**2)/(:W**5)
 !COMPUT/TABL &t :F1 = :F/(EXP(({h}*{c})/(:W*{k}*{t1})) - 1)*1e-14
 COMPUT/TABL &t :F2 = :F/(EXP(({h}*{c})/(:W*{k}*{t2})) - 1)*1e-14
 COMPUT/TABL &t :F3 = :F/(EXP(({h}*{c})/(:W*{k}*{t3})) - 1)*1e-14
 COMPUT/TABL &t :Flux = :F/(EXP(({h}*{c})/(:W*{k}*{t4})) - 1)*1e-14
!
! SET/GRAPH pm=0 font=1 yoffset=25 xa=auto ya=0,6
 SET/GRAPH colour=1 pm=0 font=1 yoffset=22 lwidth=2 tsize=1.5 xa=auto ya=0,7
! SET/GRAPH pm=0 font=1 lwidth=2 twidth=2 xa=auto ya=auto
 SET/GRAP stype=0 ltype=1
 CREATE/GRAP
!
 PLOT/TABL &t :Wavelength :Flux
 SET/GRAPH colour=2
 OVERPLOT/TABL &t :Wavelength :Flux
 OVERPLOT/TABL &t :Wavelength :F3
 OVERPLOT/TABL &t :Wavelength :F2
! OVERPLOT/TABL &t :Wavelength :F1
 SET/GRAPH colour=1
!
 OVERPLOT/LINE 2 700,0 700,7
 OVERPLOT/LINE 2 400,0 400,7
! 
! LABEL/GRAPH "Black Body Radiation"
 LABEL/GRAPH "I-R"
 LABEL/GRAPH "Visible"
 LABEL/GRAPH "U-V"
!
! LABEL/GRAPH "3000 K"
 LABEL/GRAPH "4700K (Arcturus)"
 LABEL/GRAPH "5780K (Sun)"
 LABEL/GRAPH "7000K (Procyon)"
!
 askToContinue:
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "Creating a postscript file [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   COPY/GRAPH {printer}
   WRITE/OUT "Converting plot to postscript file three_blackbodies.ps"
   $mv {printer}.ps three_blackbodies.ps
   GOTO continue
 ELSEIF reply(:1) .eq. "n" THEN
   GOTO continue
 ELSE
   GOTO askToContinue
 ENDIF
 continue:
!
 askRemove:
 WRITE/KEYW reply/c/1/1 y
 INQUIRE/KEYW reply "Remove all temporary files [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO remove
 ELSEIF reply(:1) .eq. "n" THEN
   ECHO/OFF
   RETURN/EXIT
 ELSE
   GOTO askRemove
 ENDIF
 remove:
!
! Clean all temporary files
!
 WRITE/OUT "Cleaning temporary files and finishing"
 $rm -f graph_wn*.plt
 $rm -f middummt.tbl
 ECHO/OFF
